@@ -225,7 +225,6 @@ struct _ClutterGstVideoSinkPrivate
 {
   CoglContext *ctx;
   CoglPipeline *template_pipeline;
-  CoglPipeline *pipeline;
   ClutterGstFrame *clt_frame;
 
   CoglTexture *frame[3];
@@ -693,8 +692,20 @@ static void
 clutter_gst_video_sink_navigation_send_event (GstNavigation *navigation,
                                               GstStructure  *structure)
 {
-  // TODO: how do we feed the events back to the UI layer? New
-  // signals?
+  GstEvent *event;
+  GstPad *pad = NULL;
+
+  event = gst_event_new_navigation (structure);
+
+  pad = gst_pad_get_peer (GST_VIDEO_SINK_PAD (navigation));
+
+  if (GST_IS_PAD (pad))
+    {
+      gst_pad_send_event (pad, event);
+      gst_object_unref (pad);
+    }
+  else
+    gst_event_unref (event);
 }
 
 static void
@@ -990,10 +1001,10 @@ dirty_default_pipeline (ClutterGstVideoSink *sink)
 {
   ClutterGstVideoSinkPrivate *priv = sink->priv;
 
-  if (priv->pipeline)
+  if (priv->clt_frame != NULL)
     {
-      cogl_object_unref (priv->pipeline);
-      priv->pipeline = NULL;
+      g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
+      priv->clt_frame = NULL;
       priv->had_upload_once = FALSE;
     }
 }
@@ -2145,12 +2156,6 @@ clutter_gst_video_sink_dispose (GObject *object)
     priv->renderer = NULL;
   }
 
-  if (priv->pipeline)
-    {
-      cogl_object_unref (priv->pipeline);
-      priv->pipeline = NULL;
-    }
-
   if (priv->clt_frame)
     {
       g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
@@ -2514,30 +2519,43 @@ ClutterGstFrame *
 clutter_gst_video_sink_get_frame (ClutterGstVideoSink *sink)
 {
   ClutterGstVideoSinkPrivate *priv;
-  CoglPipeline *pipeline;
 
   g_return_val_if_fail (CLUTTER_GST_IS_VIDEO_SINK (sink), NULL);
 
-  priv = sink->priv;
-
-  pipeline = clutter_gst_video_sink_get_pipeline (sink);
-
-  if (pipeline == NULL)
+  if (!clutter_gst_video_sink_is_ready (sink))
     return NULL;
 
-  if (priv->clt_frame != NULL && priv->clt_frame->pipeline != pipeline)
-    {
-      g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
-      priv->clt_frame = NULL;
-    }
+  priv = sink->priv;
 
   if (priv->clt_frame == NULL)
     {
       priv->clt_frame = clutter_gst_frame_new ();
-      priv->clt_frame->pipeline = cogl_object_ref (pipeline);
       clutter_gst_video_resolution_from_video_info (&priv->clt_frame->resolution,
                                                     &priv->info);
+      priv->clt_frame->pipeline = cogl_pipeline_new (priv->ctx);
+      clutter_gst_video_sink_setup_pipeline (sink, priv->clt_frame->pipeline);
+      clutter_gst_video_sink_attach_frame (sink, priv->clt_frame->pipeline);
+      priv->balance_dirty = FALSE;
     }
+  else if (priv->balance_dirty)
+    {
+      g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
+
+      priv->clt_frame = clutter_gst_frame_new ();
+      clutter_gst_video_resolution_from_video_info (&priv->clt_frame->resolution,
+                                                    &priv->info);
+      priv->clt_frame->pipeline = cogl_pipeline_new (priv->ctx);
+
+      clutter_gst_video_sink_setup_pipeline (sink, priv->clt_frame->pipeline);
+      clutter_gst_video_sink_attach_frame (sink, priv->clt_frame->pipeline);
+      priv->balance_dirty = FALSE;
+    }
+  else if (priv->frame_dirty)
+    {
+      clutter_gst_video_sink_attach_frame (sink, priv->clt_frame->pipeline);
+    }
+
+  priv->frame_dirty = FALSE;
 
   return priv->clt_frame;
 }
@@ -2570,39 +2588,10 @@ clutter_gst_video_sink_get_pipeline (ClutterGstVideoSink *sink)
 
   g_return_val_if_fail (CLUTTER_GST_IS_VIDEO_SINK (sink), NULL);
 
-  if (!clutter_gst_video_sink_is_ready (sink))
+  if (clutter_gst_video_sink_get_frame (sink) == NULL)
     return NULL;
 
-  priv = sink->priv;
-
-  if (priv->pipeline == NULL)
-    {
-      priv->pipeline = cogl_pipeline_new (priv->ctx);
-      clutter_gst_video_sink_setup_pipeline (sink, priv->pipeline);
-      clutter_gst_video_sink_attach_frame (sink, priv->pipeline);
-      priv->balance_dirty = FALSE;
-    }
-  else if (priv->balance_dirty)
-    {
-      cogl_object_unref (priv->pipeline);
-      priv->pipeline = cogl_pipeline_new (priv->ctx);
-
-      clutter_gst_video_sink_setup_pipeline (sink, priv->pipeline);
-      clutter_gst_video_sink_attach_frame (sink, priv->pipeline);
-      priv->balance_dirty = FALSE;
-    }
-  else if (priv->frame_dirty)
-    {
-      CoglPipeline *pipeline = cogl_pipeline_copy (priv->pipeline);
-      cogl_object_unref (priv->pipeline);
-      priv->pipeline = pipeline;
-
-      clutter_gst_video_sink_attach_frame (sink, pipeline);
-    }
-
-  priv->frame_dirty = FALSE;
-
-  return priv->pipeline;
+  return priv->clt_frame->pipeline;
 }
 
 /**
@@ -2634,9 +2623,9 @@ clutter_gst_video_sink_setup_pipeline (ClutterGstVideoSink *sink,
 
   if (priv->renderer)
     {
-      clutter_gst_video_sink_setup_conversions (sink, priv->pipeline);
-      clutter_gst_video_sink_setup_balance (sink, priv->pipeline);
-      priv->renderer->setup_pipeline (sink, priv->pipeline);
+      clutter_gst_video_sink_setup_conversions (sink, pipeline);
+      clutter_gst_video_sink_setup_balance (sink, pipeline);
+      priv->renderer->setup_pipeline (sink, pipeline);
     }
 }
 
